@@ -78,7 +78,7 @@ impl<'a> Block<'a> {
 }
 
 impl<'a> Iterator for Block<'a> {
-    type Item = u32;
+    type Item = i64;
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.remaining == 0 {
@@ -97,11 +97,11 @@ impl<'a> Iterator for Block<'a> {
             self.advance_miniblock();
         }
 
-        Some(result as u32)
+        Some(result)
     }
 }
 
-/// Decoder of parquets' `DELTA_BINARY_PACKED`. Implements `Iterator<Item = i32>`.
+/// Decoder of parquets' `DELTA_BINARY_PACKED`. Implements `Iterator<Item = i64>`.
 /// # Implementation
 /// This struct does not allocate on the heap.
 #[derive(Debug)]
@@ -109,10 +109,10 @@ pub struct Decoder<'a> {
     block_size: u64,
     num_mini_blocks: usize,
     values_per_mini_block: usize,
-    total_count: usize, // total number of elements
-    first_value: i64,   // the cumulative
+    values_remaining: usize,
+    next_value: i64,
     values: &'a [u8],
-    current_block: Block<'a>,
+    current_block: Option<Block<'a>>,
     // the total number of bytes consumed up to a given point, excluding the bytes on the current_block
     consumed_bytes: usize,
 }
@@ -139,18 +139,24 @@ impl<'a> Decoder<'a> {
         let values_per_mini_block = block_size as usize / num_mini_blocks;
         assert_eq!(values_per_mini_block % 8, 0);
 
-        let current_block = Block::new(
-            values,
-            num_mini_blocks as usize,
-            values_per_mini_block,
-            total_count,
-        );
+        // If we only have one value (first_value), there are no blocks.
+        let current_block = if total_count > 1 {
+            Some(Block::new(
+                values,
+                num_mini_blocks,
+                values_per_mini_block,
+                total_count - 1,
+            ))
+        } else {
+            None
+        };
+
         Self {
             block_size,
             num_mini_blocks,
             values_per_mini_block,
-            total_count,
-            first_value,
+            values_remaining: total_count,
+            next_value: first_value,
             values,
             current_block,
             consumed_bytes,
@@ -159,41 +165,53 @@ impl<'a> Decoder<'a> {
 
     /// Returns the total number of bytes consumed up to this point by [`Decoder`].
     pub fn consumed_bytes(&self) -> usize {
-        self.consumed_bytes + self.current_block.consumed_bytes
+        self.consumed_bytes + self.current_block.as_ref().map_or(0, |b| b.consumed_bytes)
     }
 }
 
 impl<'a> Iterator for Decoder<'a> {
-    type Item = i32;
+    type Item = i64;
 
     fn next(&mut self) -> Option<Self::Item> {
-        if self.total_count == 0 {
+        if self.values_remaining == 0 {
             return None;
         }
-        let delta = if let Some(x) = self.current_block.next() {
-            x as i64
+
+        let result = Some(self.next_value);
+
+        self.values_remaining -= 1;
+        if self.values_remaining == 0 {
+            return result;
+        }
+
+        // At this point we must have at least one block and value available
+        let current_block = self.current_block.as_mut().unwrap();
+        let delta = if let Some(x) = current_block.next() {
+            x
         } else {
             // load next block
-            self.values = &self.values[self.current_block.consumed_bytes..];
-            self.consumed_bytes += self.current_block.consumed_bytes;
-            self.current_block = Block::new(
+            self.values = &self.values[current_block.consumed_bytes..];
+            self.consumed_bytes += current_block.consumed_bytes;
+
+            let mut next_block = Block::new(
                 self.values,
                 self.num_mini_blocks,
                 self.values_per_mini_block,
-                self.total_count,
+                self.values_remaining,
             );
-            // block is never empty because `self.total_count > 0` at this point, so this is infalible
-            self.current_block.next().unwrap() as i64
-        };
-        self.total_count -= 1;
 
-        let result = Some(self.first_value as i32);
-        self.first_value += delta;
+            let delta = next_block.next().unwrap();
+            self.current_block = Some(next_block);
+
+            delta
+        };
+
+        self.next_value += delta;
         result
     }
 
     fn size_hint(&self) -> (usize, Option<usize>) {
-        (self.total_count as usize, Some(self.total_count as usize))
+        (self.values_remaining, Some(self.values_remaining))
     }
 }
 
@@ -201,9 +219,27 @@ impl<'a> Iterator for Decoder<'a> {
 mod tests {
     use super::*;
 
+    #[test]
+    fn single_value() {
+        // Generated by parquet-rs
+        //
+        // header: [128, 1, 4, 1, 2]
+        // block size: 128, 1
+        // mini-blocks: 4
+        // elements: 1
+        // first_value: 2 <=z> 1
+        let data = &[128, 1, 4, 1, 2];
+
+        let mut decoder = Decoder::new(data);
+        let r = decoder.by_ref().collect::<Vec<_>>();
+
+        assert_eq!(&r[..], &[1]);
+        assert_eq!(decoder.consumed_bytes(), 5);
+    }
+
     #[test]
     fn test_from_spec() {
-        let expected = (1i32..=5).collect::<Vec<_>>();
+        let expected = (1..=5).collect::<Vec<_>>();
         // VALIDATED FROM SPARK==3.1.1
         // header: [128, 1, 4, 5, 2]
         // block size: 128, 1
@@ -225,7 +261,7 @@ mod tests {
 
     #[test]
     fn case2() {
-        let expected = vec![1i32, 2, 3, 4, 5, 1];
+        let expected = vec![1, 2, 3, 4, 5, 1];
         // VALIDATED FROM SPARK==3.1.1
         // header: [128, 1, 4, 6, 2]
         // block size: 128, 1 <=u> 128
@@ -264,8 +300,8 @@ mod tests {
             100,    // first_value <=z> 50
 
             // Block 1 header: [7, 3, 4, 0, 0]
-            7,          // min_delta <=z> -4
-            3, 4, 0, 0, // bit_widths [3, 4, 0, 0]
+            7,            // min_delta <=z> -4
+            3, 4, 255, 0, // bit_widths (255 should not be used as only two miniblocks are needed)
 
             // 32 3-bit values of 0 for mini-block 1 (12 bytes)
             0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
